import re
from enum import Enum


def camel_dict_to_snake_dict(source: {}) -> {}:
    pattern = re.compile(r'(?<!^)(?=[A-Z])')
    return {pattern.sub('_', key).lower(): list(camel_dict_to_snake_dict(in_value) for in_value in value
                                                ) if isinstance(value, type([])) else value for key, value
            in source.items()}


def snake_dict_to_camel_dict(source: {}) -> {}:
    return {''.join(word.title() for word in key.split('_')): value.name.title() if isinstance(value, Enum) else value
            for key, value in source.items()}



import abc
import traceback

from typing import TYPE_CHECKING
if TYPE_CHECKING:
    from balticlsc.computation_module.baltic_lsc.data_handler import IDataHandler, DataHandler
    from balticlsc.computation_module.baltic_lsc.job_registry import IJobRegistry, JobRegistry

from balticlsc.computation_module.data_model.messages import Status
from balticlsc.computation_module.utils.logger import logger


class TokenListener:

    @classmethod
    def __subclasshook__(cls, subclass):
        return (hasattr(subclass, 'data_received') and
                callable(subclass.data_received) and
                hasattr(subclass, 'optional_data_received') and
                callable(subclass.optional_data_received) and
                hasattr(subclass, 'data_ready') and
                callable(subclass.data_ready) and
                hasattr(subclass, 'data_complete') and
                callable(subclass.data_complete) or
                NotImplemented)

    @abc.abstractmethod
    def __init__(self, registry: 'IJobRegistry', data: 'IDataHandler'):
        self._data = data
        self._registry = registry

    @abc.abstractmethod
    def data_received(self, pin_name: str):
        pass

    @abc.abstractmethod
    def optional_data_received(self, pin_name: str):
        pass

    @abc.abstractmethod
    def data_ready(self):
        pass

    @abc.abstractmethod
    def data_complete(self):
        pass


class JobThread:

    def __init__(self, pin_name: str, listener: TokenListener, registry: 'JobRegistry', handler: 'DataHandler'):
        self.__pin_name = pin_name
        self.__listener = listener
        self.__registry = registry
        self.__handler = handler

    def run(self):
        try:
            self.__listener.data_received(self.__pin_name)
            if 'true' == self.__registry.get_pin_configuration(self.__pin_name).is_required:
                self.__listener.optional_data_received(self.__pin_name)

            pin_aggregated_status = Status.COMPLETED

            for pin_name in self.__registry.get_strong_pin_names():
                pin_status = self.__registry.get_pin_status(pin_name)
                if Status.WORKING == pin_status:
                    pin_aggregated_status = Status.WORKING
                elif Status.IDLE == pin_status:
                    pin_aggregated_status = Status.IDLE
                    break

            if Status.IDLE != pin_aggregated_status:
                self.__listener.data_ready()
            if Status.COMPLETED == pin_aggregated_status:
                self.__listener.data_complete()
        except BaseException as e:
            logger.info('Run module error: : ' + str(e))
            logger.info(traceback.format_exc())
            self.__handler.fail_processing(str(e))



import os.path
from os import listdir
from os.path import isfile, join
from typing import List, Tuple
import face_recognition
from matplotlib import pyplot, patches
from PIL import Image
import numpy as np
from balticlsc.computation_module.api_access.job_controller import init_job_controller, TokenListener
from balticlsc.computation_module.baltic_lsc.data_handler import IDataHandler
from balticlsc.computation_module.baltic_lsc.job_registry import IJobRegistry
from balticlsc.computation_module.data_model.messages import Status
from balticlsc.computation_module.utils.logger import logger


class MyTokenListener(TokenListener):

    def __init__(self, registry: IJobRegistry, data: IDataHandler):
        super().__init__(registry, data)

    def data_received(self, pin_name: str):
        # Place your code here:
        pass

    def optional_data_received(self, pin_name: str):
        # Place your code here:
        pass

    def data_ready(self):
        # Place your code here:
        pass

    def data_complete(self):
        self._registry.set_status(Status.WORKING)
        filepath = self._data.obtain_data_item('Input')
        filename = os.path.basename(filepath)

        if not filename.lower().endswith(('.png', '.jpg', '.jpeg', '.tiff', '.bmp', '.gif')):
            msg = 'wrong format of the file "' + filename + '", omitting'
            logger.warning(msg)
            raise ValueError(msg)

        # Mark faces and save the image
        image = np.array(Image.open(filepath))
        im = Image.fromarray(image)
        im.save(filepath)
        height: int = image.shape[0]
        width: int = image.shape[1]
        dpi: int = 100
        faces_coords: List[Tuple[int]] = face_recognition.face_locations(image)
        figure = pyplot.figure(frameon=False, dpi=dpi)
        figure.set_size_inches(width / dpi, height / dpi)
        ax = pyplot.Axes(figure, [0., 0., 1., 1.])
        ax.set_axis_off()
        figure.add_axes(ax)
        ax.imshow(image)
        logger.info('adding ' + str(len(faces_coords)) + ' faces to image "' + filename + '"')
        fig = pyplot.gcf()
        fig.savefig(fname=filepath, dpi=dpi, bbox_inches='tight')

        for index in range(len(faces_coords)):
            x_start = faces_coords[index][3]
            y_start = faces_coords[index][0]
            x_width = (faces_coords[index][1] - faces_coords[index][3])
            y_height = (faces_coords[index][2] - faces_coords[index][0])
            rect = patches.Rectangle((x_start, y_start), x_width, y_height,
                                     edgecolor='r', facecolor="none")
            ax.add_patch(rect)

        pyplot.savefig(fname=filepath, dpi=dpi, bbox_inches='tight')
        pyplot.close()
        self._data.send_data_item('Output', filepath, True)
        self._data.finish_processing()


app = init_job_controller(MyTokenListener)



import json
import os
import threading
import traceback
from typing import Type, Optional
from flask import Flask, request, Response
from balticlsc.computation_module.baltic_lsc.data_handler import DataHandler
from balticlsc.computation_module.baltic_lsc.job_registry import JobRegistry
from balticlsc.computation_module.baltic_lsc.job_engine import TokenListener, JobThread
from balticlsc.computation_module.utils.logger import logger
from balticlsc.computation_module.data_model.messages import InputTokenMessage, SeqToken
from balticlsc.computation_module.data_model.pins_configuration import get_pins_configuration
from balticlsc.computation_module.utils.utils import camel_dict_to_snake_dict, snake_dict_to_camel_dict

__registry: Optional[JobRegistry] = None
__handler: Optional[DataHandler] = None
__listener_type: Optional[Type[TokenListener]] = None


def init_job_controller(listener_type: Type[TokenListener]) -> Flask:
    global __listener_type, __registry, __handler
    __listener_type = listener_type
    pins_configuration = get_pins_configuration()
    __registry = JobRegistry(pins_configuration)
    __handler = DataHandler(__registry, pins_configuration)
    app = Flask(os.getenv('SYS_MODULE_NAME', 'BalticLSC module'))

    @app.route('/token', methods=['POST'])
    def process_token():
        try:
            logger.debug('Token message received: ' + str(request.json))
            input_token = InputTokenMessage(
                **{key: value if 'token_seq_stack' != key else list(
                    SeqToken(**in_value) for in_value in value)
                   for key, value in camel_dict_to_snake_dict(request.json).items() if key in
                   InputTokenMessage.__dict__['__annotations__']})
            __registry.register_token(input_token)
            try:
                result = __handler.check_connection(input_token.pin_name, json.loads(input_token.values))
                if 0 == result:
                    logger.debug(f'Running job for pin "{input_token.pin_name}"')
                    job_thread = JobThread(input_token.pin_name, __listener_type(__registry, __handler),
                                           __registry, __handler)
                    __registry.register_thread(job_thread)
                    pin_task = threading.Thread(target=job_thread.run)
                    pin_task.daemon = True
                    pin_task.start()
                    return Response(status=200, mimetype='application/json')
                if - 1 == result:
                    ret__message = 'No response (' + input_token.pin_name + ').'
                    logger.debug(ret__message)
                    return Response(ret__message, status=404, mimetype='application/json')
                if - 2 == result:
                    ret__message = 'Unauthorized (' + input_token.pin_name + ').'
                    logger.debug(ret__message)
                    return Response(ret__message, status=401, mimetype='application/json')
                if - 3 == result:
                    ret__message = 'Invalid path (' + input_token.pin_name + ').'
                    logger.debug(ret__message)
                    return Response(ret__message, status=401, mimetype='application/json')
                return Response(status=400, mimetype='application/json')
            except BaseException as e:
                logger.debug('Corrupted token: : ' + str(e))
                logger.debug(traceback.format_exc())
                return Response('Error of type ' + type(e).__name__ + ':' + str(e), status=200,
                                mimetype='application/json')
        except BaseException as e:
            logger.debug('Corrupted token: : ' + str(e))
            logger.debug(traceback.format_exc())
            return Response(str(e), status=400, mimetype='application/json')

    @app.route('/status', methods=['GET'])
    def get_status():
        return Response(json.dumps(snake_dict_to_camel_dict(__registry.get_job_status().__dict__)), status=200,
                        mimetype='application/json')

    return app



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="balticlsc",
    version="0.1.3",
    author="BalticLSC",
    author_email="Kamil.Rybinski@pw.edu.pl",
    description="Baltic LSC module old_scheme",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/balticlsc/Computation_Module_Template_Python",
    install_requires=[
        'flask',
        'requests',
        'pymongo'
    ],
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.10',
)
